/*
  TinyTyperBlind Sketch for ATMega328-Based Arduinos
  Written by 'starhawk' <laserhawk64@gmail.com> (Please do not spam, I get enough of that already,
  and Google no longer cares!)

  Based *HEAVILY* on the advcodetest' ( "Advanced Code Test", one presumes... ) example code and
  PS2KeyAdvanced Library for Arduino, released under Lesser GNU Public License Version 2.1.
  Written by Paul Carpenter, PC Services <sales@pcserviceselectronics.co.uk>

  --------------------------------------------------

  This code ( "TinyTyperBlind.ino" aka ""TinyTyperBlind" ) is distributed under a 3-Clause /
  "Revised" BSD License, as provided here:

      Copyright 2021 Christopher Havel alias 'starhawk'

      Redistribution and use in source and binary forms, with or without modification, are permitted
      provided that the following conditions are met:

      1. Redistributions of source code must retain the above copyright notice, this list of conditions
         and the following disclaimer.
      2. Redistributions in binary form must reproduce the above copyright notice, this list of
         conditions and the following disclaimer in the documentation and/or other materials provided 
         with the distribution.
      3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse
         or promote products derived from this software without specific prior written permission.

      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
      WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
      PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
      ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
      LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
      INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
      TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
      ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

  --------------------------------------------------

  The PS2KeyAdvanced library, where bundled with the TinyTyperBlind.ino Sketch code, carries the licensing that
  it is otherwise normally bundled with otherwise. Currently that is, as noted, the Lesser GPL Ver. 2.1 License.
  
*/

#include <PS2KeyAdvanced.h>

/* Keyboard constants  Change to suit your Arduino
   define pins used for data and clock from keyboard */
#define DATAPIN 4
#define IRQPIN  3

uint16_t c;
uint8_t breaks = 0;
uint8_t repeats = 0;

PS2KeyAdvanced keyboard;

byte initprnt[3] = {0x27, 0x33, 0x32};
byte lineprnt[1] = {0x0D};
byte cutpaper[1] = {0x0C};

void setup( )
{

// init printer and welcome message
// 26 chars/line sucks!
Serial.begin( 9600 );
Serial.write(initprnt, 3);
Serial.print( "TinyTyper NoLCD Version" );
Serial.write(lineprnt, 1);
Serial.write(lineprnt, 1);
Serial.print( "You are now ready to type." );
Serial.write(lineprnt, 1);
Serial.print( "You have 26 characters per" );
Serial.write(lineprnt, 1);
Serial.print( "line. However, you can only" );
Serial.write(lineprnt, 1);
Serial.print( "see what you've typed once" );
Serial.write(lineprnt, 1);
Serial.print( "you have typed an entire" );
Serial.write(lineprnt, 1);
Serial.print( "line, because of how the" );
Serial.write(lineprnt, 1);
Serial.print( "printer works. (Sorry!)" );
Serial.write(lineprnt, 1);
Serial.print( "Word wrap is automatic, at" );
Serial.write(lineprnt, 1);
Serial.print( "least. 'ENTER' prints the" );
Serial.write(lineprnt, 1);
Serial.print( "current line and starts a" );
Serial.write(lineprnt, 1);
Serial.print( "new one early, and 'ESC'" );
Serial.write(lineprnt, 1);
Serial.print( "prints the current line," );
Serial.write(lineprnt, 1);
Serial.print( "cuts the paper, and stops." );
Serial.write(lineprnt, 1);

// Configure the keyboard library
keyboard.begin( DATAPIN, IRQPIN );
/* assumes keyboard is present, let's not assume the
 user is a complete and utter dunderhead, please! */
}

void loop( )
{
// read the next key
c = keyboard.read( );
if( ( c & 0xFF ) > 0 )
  Serial.println( c , HEX );
/* now do something with keys entered results on serial monitor */
if( !( c & PS2_BREAK ) )  // only on make codes
  switch( c & 0xFF )
    {
      case PS2_KEY_ENTER:
                   Serial.write(lineprnt, 1);
                   break;
      case PS2_KEY_ESC:
                   Serial.write(cutpaper, 1);
                   break;
      case PS2_KEY_F1 || PS2_KEY_F2 || PS2_KEY_F3 || PS2_KEY_F4 || PS2_KEY_F5 || PS2_KEY_F6 || PS2_KEY_F7 || PS2_KEY_F8 ||
           PS2_KEY_F9 || PS2_KEY_F10 || PS2_KEY_F11 || PS2_KEY_F12 || PS2_KEY_F13 || PS2_KEY_F14 || PS2_KEY_F15 || PS2_KEY_F16 ||
           PS2_KEY_F17 || PS2_KEY_F18 || PS2_KEY_F19 || PS2_KEY_F20 || PS2_KEY_F21 || PS2_KEY_F22 || PS2_KEY_F23 || PS2_KEY_F24 ||
           PS2_KEY_SYSRQ || PS2_KEY_BS || PS2_KEY_TAB || PS2_KEY_INTL1 || PS2_KEY_INTL2 || PS2_KEY_INTL3 || PS2_KEY_INTL4 ||
           PS2_KEY_INTL5 || PS2_KEY_LANG1 || PS2_KEY_LANG2 || PS2_KEY_LANG3 || PS2_KEY_LANG4 || PS2_KEY_LANG5 || PS2_KEY_PRTSCR ||
           PS2_KEY_L_CTRL || PS2_KEY_R_CTRL || PS2_KEY_L_ALT || PS2_KEY_R_ALT || PS2_KEY_L_GUI || PS2_KEY_R_GUI || PS2_KEY_MENU ||
           PS2_KEY_HOME || PS2_KEY_END || PS2_KEY_PGUP || PS2_KEY_PGDN || PS2_KEY_L_ARROW || PS2_KEY_R_ARROW || PS2_KEY_UP_ARROW ||
           PS2_KEY_DN_ARROW || PS2_KEY_INSERT || PS2_KEY_DELETE || PS2_KEY_NEXT_TR || PS2_KEY_PREV_TR || PS2_KEY_STOP ||
           PS2_KEY_PLAY || PS2_KEY_MUTE || PS2_KEY_VOL_UP || PS2_KEY_VOL_DN || PS2_KEY_MEDIA || PS2_KEY_EMAIL || PS2_KEY_CALC ||
           PS2_KEY_COMPUTER || PS2_KEY_WEB_SEARCH || PS2_KEY_WEB_HOME || PS2_KEY_WEB_BACK || PS2_KEY_WEB_FORWARD ||
           PS2_KEY_WEB_STOP || PS2_KEY_WEB_REFRESH || PS2_KEY_WEB_FAVOR || PS2_KEY_POWER || PS2_KEY_SLEEP || PS2_KEY_WAKE:
                   break;
    }
}
